@@ -2,6 +2,7 @@
  *
  * Copyright (C) 2001 Jonathan Blandford <jrb@alum.mit.edu>
  * Copyright (C) 2006 Emmanuele Bassi <ebassi@gnome.org>
+ * Copyright (C) 2008 Cosimo Cecchi <cosimoc@gnome.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,10 +25,9 @@
 /* MAYBE I LIED... -jrb */
 
 #include <config.h>
-#include <gnome.h>
 #include <gconf/gconf-client.h>
-#include <glade/glade.h>
 #include <gdk/gdkx.h>
+#include <gdk/gdkkeysyms.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
@@ -37,9 +37,8 @@
 #include <errno.h>
 #include <locale.h>
 #include <glib/gi18n.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
-#include <libgnomevfs/gnome-vfs-async-ops.h>
-#include <libgnomevfs/gnome-vfs-ops.h>
+#include <gio/gio.h>
+#include <pwd.h>
 #include <X11/Xutil.h>
 
 #include "screenshot-shadow.h"
@@ -52,8 +51,11 @@
 
 #define GNOME_SCREENSHOT_GCONF  "/apps/gnome-screenshot"
 #define INCLUDE_BORDER_KEY      GNOME_SCREENSHOT_GCONF "/include_border"
+#define INCLUDE_POINTER_KEY     GNOME_SCREENSHOT_GCONF "/include_pointer"
 #define LAST_SAVE_DIRECTORY_KEY GNOME_SCREENSHOT_GCONF "/last_save_directory"
 #define BORDER_EFFECT_KEY       GNOME_SCREENSHOT_GCONF "/border_effect"
+#define DELAY_KEY               GNOME_SCREENSHOT_GCONF "/delay"
+
 
 enum
 {
@@ -70,6 +72,22 @@ typedef enum {
   SCREENSHOT_EFFECT_BORDER
 } ScreenshotEffectType;
 
+typedef enum
+{
+  TEST_LAST_DIR = 0,
+  TEST_DESKTOP = 1,
+  TEST_TMP = 2,
+} TestType;
+
+typedef struct 
+{
+  char *base_uris[3];
+  char *retval;
+  int iteration;
+  TestType type;
+  GdkWindow *window;
+} AsyncExistenceJob;
+
 static GdkPixbuf *screenshot = NULL;
 
 /* Global variables*/
@@ -81,33 +99,35 @@ static gboolean save_immediately = FALSE;
 /* Options */
 static gboolean take_window_shot = FALSE;
 static gboolean include_border = FALSE;
+static gboolean include_pointer = TRUE;
 static char *border_effect = NULL;
 static guint delay = 0;
 
 /* some local prototypes */
-static void display_help           (GtkWindow *parent);
-static void save_done_notification (gpointer   data);
-static char *get_desktop_dir (void);
+static void  display_help           (GtkWindow *parent);
+static void  save_done_notification (gpointer   data);
+static char *get_desktop_dir        (void);
+static void  save_options           (void);
 
 static GtkWidget *border_check = NULL;
 static GtkWidget *effect_combo = NULL;
+static GtkWidget *effect_label = NULL;
 
 static void
 display_help (GtkWindow *parent)
 {
   GError *error = NULL;
 
-  gnome_help_display_desktop (NULL,
-                              "user-guide",
-			      "user-guide.xml",
-                              "goseditmainmenu-53",
-			      &error);
+  gtk_show_uri (gtk_window_get_screen (parent),
+		"ghelp:user-guide#goseditmainmenu-53",
+		gtk_get_current_event_time (), &error);
 
   if (error)
     {
       screenshot_show_gerror_dialog (parent,
                                      _("Error loading the help page"),
                                      error);
+      g_error_free (error);
     }
 }
 
@@ -140,6 +160,7 @@ target_toggled_cb (GtkToggleButton *button,
       
       gtk_widget_set_sensitive (border_check, take_window_shot);
       gtk_widget_set_sensitive (effect_combo, take_window_shot);
+      gtk_widget_set_sensitive (effect_label, take_window_shot);
     }
 }
 
@@ -156,6 +177,13 @@ include_border_toggled_cb (GtkToggleButton *button,
   include_border = gtk_toggle_button_get_active (button);
 }
 
+static void
+include_pointer_toggled_cb (GtkToggleButton *button,
+                            gpointer         data)
+{
+  include_pointer = gtk_toggle_button_get_active (button);
+}
+
 static void
 effect_combo_changed_cb (GtkComboBox *combo,
                          gpointer     user_data)
@@ -177,6 +205,18 @@ effect_combo_changed_cb (GtkComboBox *combo,
     }
 }
 
+static gint 
+key_press_cb (GtkWidget* widget, GdkEventKey* event, gpointer data)
+{
+  if (event->keyval == GDK_F1)
+    {
+      display_help (GTK_WINDOW (widget));
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
 typedef struct {
   ScreenshotEffectType id;
   const gchar *label;
@@ -285,21 +325,23 @@ create_effects_frame (GtkWidget   *outer_vbox,
   gtk_widget_show (hbox);
 
   align = gtk_alignment_new (0.0, 0.0, 0.0, 0.0);
-  gtk_widget_set_size_request (align, 48, -1);
+  gtk_alignment_set_padding (GTK_ALIGNMENT (align), 0, 0, 12, 0);
   gtk_box_pack_start (GTK_BOX (hbox), align, FALSE, FALSE, 0);
   gtk_widget_show (align);
 
-#if 0
-  image = gtk_image_new_from_stock (SCREENSHOOTER_ICON,
-                                    GTK_ICON_SIZE_DIALOG);
-  gtk_container_add (GTK_CONTAINER (align), image);
-  gtk_widget_show (image);
-#endif
-
   vbox = gtk_vbox_new (FALSE, 6);
-  gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE, 0);
+  gtk_container_add (GTK_CONTAINER (align), vbox);
   gtk_widget_show (vbox);
 
+  /** Include pointer **/
+  check = gtk_check_button_new_with_mnemonic (_("Include _pointer"));
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), include_pointer);
+  g_signal_connect (check, "toggled",
+                    G_CALLBACK (include_pointer_toggled_cb),
+                    NULL);
+  gtk_box_pack_start (GTK_BOX (vbox), check, FALSE, FALSE, 0);
+  gtk_widget_show (check);
+
   /** Include window border **/
   check = gtk_check_button_new_with_mnemonic (_("Include the window _border"));
   gtk_widget_set_sensitive (check, take_window_shot);
@@ -317,9 +359,11 @@ create_effects_frame (GtkWidget   *outer_vbox,
   gtk_widget_show (hbox);
 
   label = gtk_label_new_with_mnemonic (_("Apply _effect:"));
+  gtk_widget_set_sensitive (label, take_window_shot);
   gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
   gtk_widget_show (label);
+  effect_label = label;
 
   combo = create_effects_combo ();
   gtk_widget_set_sensitive (combo, take_window_shot);
@@ -413,7 +457,7 @@ create_screenshot_frame (GtkWidget   *outer_vbox,
   adjust = GTK_ADJUSTMENT (gtk_adjustment_new ((gdouble) delay,
                                                0.0, 99.0,
                                                1.0,  1.0,
-                                               1.0));
+                                               0.0));
   spin = gtk_spin_button_new (adjust, 1.0, 0);
   g_signal_connect (spin, "value-changed",
                     G_CALLBACK (delay_spin_value_changed_cb),
@@ -439,10 +483,9 @@ create_interactive_dialog (void)
 
   retval = gtk_dialog_new ();
   gtk_window_set_resizable (GTK_WINDOW (retval), FALSE);
-  gtk_window_set_skip_taskbar_hint (GTK_WINDOW (retval), TRUE);
   gtk_container_set_border_width (GTK_CONTAINER (retval), 5);
   gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (retval)->vbox), 2);
-  gtk_window_set_title (GTK_WINDOW (retval), "");
+  gtk_window_set_title (GTK_WINDOW (retval), _("Take Screenshot"));
 
   /* main container */
   main_vbox = gtk_vbox_new (FALSE, 18);
@@ -462,6 +505,8 @@ create_interactive_dialog (void)
                           _("Take _Screenshot"), GTK_RESPONSE_OK,
                           NULL);
 
+  gtk_dialog_set_default_response (GTK_DIALOG (retval), GTK_RESPONSE_OK);
+
   /* we need to block on "response" and keep showing the interactive
    * dialog in case the user did choose "help"
    */
@@ -469,87 +514,13 @@ create_interactive_dialog (void)
                     G_CALLBACK (interactive_dialog_response_cb),
                     NULL);
 
+  g_signal_connect (G_OBJECT (retval), "key-press-event",
+                    G_CALLBACK(key_press_cb), 
+                    NULL);
+
   return retval;
 }
 
-/* We assume that uri is valid and has been tested elsewhere
- */
-static char *
-generate_filename_for_uri (const char *uri)
-{
-  char *retval;
-  char *file_name;
-  int i = 1;
-
-  if (window_title)
-    {
-      /* translators: this is the name of the file that gets made up
-       * with the screenshot if a specific window is taken */
-      file_name = g_strdup_printf (_("Screenshot-%s.png"), window_title);
-    }
-  else
-    {
-      /* translators: this is the name of the file that gets made up
-       * with the screenshot if the entire screen is taken */
-      file_name = g_strdup (_("Screenshot.png"));
-    }
-
-  retval = g_build_filename (uri, file_name, NULL);
-  g_free (file_name);
-
-  do
-    {
-      GnomeVFSFileInfo *info;
-      GnomeVFSResult result;
-
-      info = gnome_vfs_file_info_new ();
-      result = gnome_vfs_get_file_info (retval, info,
-                                        GNOME_VFS_FILE_INFO_DEFAULT |
-                                        GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
-      gnome_vfs_file_info_unref (info);
-
-      switch (result)
-	{
-	case GNOME_VFS_ERROR_NOT_FOUND:
-	  return retval;
-	case GNOME_VFS_OK:
-	  g_free (retval);
-	  break;
-	case GNOME_VFS_ERROR_PROTOCOL_ERROR:
-	  /* try again?  I'm getting these errors sporadically */
-	default:
-	  g_warning ("ERROR: %s:%s\n",
-                     retval,
-                     gnome_vfs_result_to_string (result));
-	  g_free (retval);
-	  return NULL;
-	}
-
-      /* We had a hit.  We need to make a new file */
-      if (window_title)
-	{
-	  /* translators: this is the name of the file that gets
-	   * made up with the screenshot if a specific window is
-	   * taken */
-	  file_name = g_strdup_printf (_("Screenshot-%s-%d.png"),
-				       window_title, i);
-	}
-      else
-	{
-	  /* translators: this is the name of the file that gets
-	   * made up with the screenshot if the entire screen is
-	   * taken */
-	  file_name = g_strdup_printf (_("Screenshot-%d.png"), i);
-	}
-
-      retval = g_build_filename (uri, file_name, NULL);
-      g_free (file_name);
-
-      i++;
-    }
-  while (TRUE);
-}
-
 static void
 save_folder_to_gconf (ScreenshotDialog *dialog)
 {
@@ -564,40 +535,99 @@ save_folder_to_gconf (ScreenshotDialog *dialog)
 			   LAST_SAVE_DIRECTORY_KEY, folder,
                            NULL);
 
+  g_free (folder);
   g_object_unref (gconf_client);
 }
 
-static gboolean
+static void
+error_dialog_response_cb (GtkDialog *d,
+                          gint response,
+                          ScreenshotDialog *dialog)
+{
+  gtk_widget_destroy (GTK_WIDGET (d));
+  
+  screenshot_dialog_focus_entry (dialog);
+}
+
+static void
+save_callback (TransferResult result,
+               char *error_message,
+               gpointer data)
+{
+  ScreenshotDialog *dialog = data;
+  GtkWidget *toplevel;
+  
+  toplevel = screenshot_dialog_get_toplevel (dialog);
+  screenshot_dialog_set_busy (dialog, FALSE);
+
+  if (result == TRANSFER_OK)
+    {
+      save_folder_to_gconf (dialog);
+      gtk_widget_destroy (toplevel);
+      
+      /* we're done, stop the mainloop now */
+      gtk_main_quit ();
+    }
+  else if (result == TRANSFER_OVERWRITE ||
+           result == TRANSFER_CANCELLED)
+    {
+      /* user has canceled the overwrite dialog or the transfer itself, let him
+       * choose another name.
+       */
+      screenshot_dialog_focus_entry (dialog);
+    }
+  else /* result == TRANSFER_ERROR */
+    {
+      /* we had an error, display a dialog to the user and let him choose
+       * another name/location to save the screenshot.
+       */
+      GtkWidget *error_dialog;
+      char *uri;
+      
+      uri = screenshot_dialog_get_uri (dialog);
+      error_dialog = gtk_message_dialog_new (GTK_WINDOW (toplevel),
+                                       GTK_DIALOG_DESTROY_WITH_PARENT,
+                                       GTK_MESSAGE_ERROR,
+                                       GTK_BUTTONS_OK,
+                                       _("Error while saving screenshot"),
+                                       NULL);
+      /* translators: first %s is the file path, second %s is the VFS error */
+      gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (error_dialog),
+                                                _("Impossible to save the screenshot "
+                                                  "to %s.\n Error was %s.\n Please choose another "
+                                                  "location and retry."), uri, error_message);
+      gtk_widget_show (error_dialog);
+      g_signal_connect (error_dialog,
+                        "response",
+                        G_CALLBACK (error_dialog_response_cb),
+                        dialog);
+
+      g_free (uri);
+    }
+      
+}
+
+static void
 try_to_save (ScreenshotDialog *dialog,
 	     const char       *target)
 {
-  GnomeVFSResult result;
-  GnomeVFSURI *source_uri;
-  GnomeVFSURI *target_uri;
+  GFile *source_file, *target_file;
 
   g_assert (temporary_file);
 
   screenshot_dialog_set_busy (dialog, TRUE);
 
-  source_uri = gnome_vfs_uri_new (temporary_file);
-  target_uri = gnome_vfs_uri_new (target);
-
-  result = screenshot_xfer_uri (source_uri,
-				target_uri,
-				screenshot_dialog_get_toplevel (dialog));
-
-  gnome_vfs_uri_unref (source_uri);
-  gnome_vfs_uri_unref (target_uri);
-
-  screenshot_dialog_set_busy (dialog, FALSE);
-
-  if (result == GNOME_VFS_OK)
-    {
-      save_folder_to_gconf (dialog);
-      return TRUE;
-    }
-
-  return FALSE;
+  source_file = g_file_new_for_path (temporary_file);
+  target_file = g_file_new_for_uri (target);
+  
+  screenshot_xfer_uri (source_file,
+                       target_file,
+                       screenshot_dialog_get_toplevel (dialog),
+                       save_callback, dialog);
+  
+  /* screenshot_xfer_uri () holds a ref, so we can unref now */
+  g_object_unref (source_file);
+  g_object_unref (target_file);
 }
 
 static void
@@ -617,99 +647,94 @@ save_done_notification (gpointer data)
     }
 }
 
+static void
+screenshot_dialog_response_cb (GtkDialog *d,
+                               gint response_id,
+                               ScreenshotDialog *dialog)
+{
+  char *uri;
+
+  if (response_id == GTK_RESPONSE_HELP)
+    {
+      display_help (GTK_WINDOW (d));
+    }
+  else if (response_id == GTK_RESPONSE_OK)
+    {
+      uri = screenshot_dialog_get_uri (dialog);
+      if (temporary_file == NULL)
+        {
+          save_immediately = TRUE;
+          screenshot_dialog_set_busy (dialog, TRUE);
+        }
+      else
+        {
+          /* we've saved the temporary file, lets try to copy it to the
+           * correct location.
+           */
+          try_to_save (dialog, uri);
+        }
+      g_free (uri);
+    }
+  else if (response_id == SCREENSHOT_RESPONSE_COPY)
+    {
+      GtkClipboard *clipboard;
+      GdkPixbuf    *screenshot;
+
+      clipboard = gtk_clipboard_get_for_display (gtk_widget_get_display (GTK_WIDGET (d)),
+                                                 GDK_SELECTION_CLIPBOARD);
+      screenshot = screenshot_dialog_get_screenshot (dialog);
+      gtk_clipboard_set_image (clipboard, screenshot);
+    }
+  else /* dialog was canceled */
+    {
+      gtk_widget_destroy (GTK_WIDGET (d));
+      gtk_main_quit ();
+    }
+}
+                               
+
 static void
 run_dialog (ScreenshotDialog *dialog)
 {
   GtkWidget *toplevel;
-  int result;
-  int keep_going;
-  char *uri;
 
   toplevel = screenshot_dialog_get_toplevel (dialog);
-
-  do
-    {
-      keep_going = FALSE;
-      result = gtk_dialog_run (GTK_DIALOG (toplevel));
-      switch (result)
-	{
-	case GTK_RESPONSE_HELP:
-	  display_help (GTK_WINDOW (toplevel));
-	  keep_going = TRUE;
-	  break;
-	case GTK_RESPONSE_OK:
-	  uri = screenshot_dialog_get_uri (dialog);
-	  if (temporary_file == NULL)
-	    {
-	      save_immediately = TRUE;
-	      screenshot_dialog_set_busy (dialog, TRUE);
-	      keep_going = TRUE;
-	    }
-	  else
-	    {
-	      /* We've saved the temporary file.  Lets try to copy it to the
-	       * correct location */
-	      if (! try_to_save (dialog, uri))
-		keep_going = TRUE;
-	    }
-	  break;
-	default:
-	  break;
-	}
-    }
-  while (keep_going);
+  
+  gtk_widget_show (toplevel);
+  
+  g_signal_connect (toplevel,
+                    "response",
+                    G_CALLBACK (screenshot_dialog_response_cb),
+                    dialog);
 }
 
 static void
-prepare_screenshot (void)
-{
+finish_prepare_screenshot (char *initial_uri, GdkWindow *window)
+{  
   ScreenshotDialog *dialog;
-  Window win;
-  char *initial_uri;
 
-  if (!screenshot_grab_lock ())
-    exit (0);
-
-  if (take_window_shot)
-    {
-      win = screenshot_find_current_window (include_border);
-      if (win == None)
-	{
-	  take_window_shot = FALSE;
-	  win = GDK_ROOT_WINDOW ();
-	}
-      else
-	{
-	  gchar *tmp;
-
-	  window_title = screenshot_get_window_title (win);
-	  tmp = screenshot_sanitize_filename (window_title);
-	  g_free (window_title);
-	  window_title = tmp;
-	}
-    }
+  /* always disable window border for full-desktop screenshots */
+  if (!take_window_shot)
+    screenshot = screenshot_get_pixbuf (window, include_pointer, FALSE);
   else
     {
-      win = GDK_ROOT_WINDOW ();
+      screenshot = screenshot_get_pixbuf (window, include_pointer, include_border);
+
+      switch (border_effect[0])
+        {
+        case 's': /* shadow */
+          screenshot_add_shadow (&screenshot);
+          break;
+        case 'b': /* border */
+          screenshot_add_border (&screenshot);
+          break;
+        case 'n': /* none */
+        default:
+          break;
+        }
     }
 
-  screenshot = screenshot_get_pixbuf (win);
-
-  if (take_window_shot) {
-    switch (border_effect[0])
-      {
-      case 's': /* shadow */
-        screenshot_add_shadow (&screenshot);
-        break;
-      case 'b': /* border */
-        screenshot_add_border (&screenshot);
-        break;
-      case 'n': /* none */
-      default:
-        break;
-      }
-  }
-
+  /* release now the lock, it was acquired when we were finding the window */
   screenshot_release_lock ();
 
   if (screenshot == NULL)
@@ -720,38 +745,232 @@ prepare_screenshot (void)
       exit (1);
     }
 
-
-  /* If uri isn't local, we should do this async before taking the sshot */
-  initial_uri = generate_filename_for_uri (last_save_dir);
-  if (initial_uri == NULL)
-    {
-      gchar *desktop_dir;
-      /* We failed to make a new name for the last save dir.  We try again
-       * with our home dir as a fallback.  If that fails, we don't have a
-       * place to put it. */
-      desktop_dir = get_desktop_dir ();
-      initial_uri = generate_filename_for_uri (desktop_dir);
-      g_free (desktop_dir);
-
-      if (initial_uri == NULL)
-	{
-	  initial_uri = g_strdup ("file:///");
-	}
-  }
   dialog = screenshot_dialog_new (screenshot, initial_uri, take_window_shot);
   g_free (initial_uri);
 
   screenshot_save_start (screenshot, save_done_notification, dialog);
 
   run_dialog (dialog);
+}
 
+static gboolean
+check_file_done (gpointer user_data)
+{
+  char *retval;
+  GdkWindow *window;
+  AsyncExistenceJob *job = user_data;
+
+  window = job->window;
+  retval = job->retval;
+  g_free (job->base_uris[1]);
+  g_free (job->base_uris[2]);
+  g_slice_free (AsyncExistenceJob, job);
+  
+  finish_prepare_screenshot (retval, window);
+  
+  return FALSE;
+}
+
+static char *
+build_uri (AsyncExistenceJob *job)
+{
+  char *retval, *file_name;
+
+  if (window_title)
+    {
+      /* translators: this is the name of the file that gets made up
+       * with the screenshot if a specific window is taken */
+      if (job->iteration == 0)
+        {
+          file_name = g_strdup_printf (_("Screenshot-%s.png"), window_title);
+        }
+      else
+        {
+          /* translators: this is the name of the file that gets
+           * made up with the screenshot if a specific window is
+           * taken */
+          file_name = g_strdup_printf (_("Screenshot-%s-%d.png"),
+                                       window_title, job->iteration);
+        }
+    }
+  else
+    {
+      if (job->iteration == 0)
+        {
+          /* translators: this is the name of the file that gets made up
+           * with the screenshot if the entire screen is taken */
+          file_name = g_strdup (_("Screenshot.png"));
+        }
+      else
+        {
+          /* translators: this is the name of the file that gets
+           * made up with the screenshot if the entire screen is
+           * taken */
+          file_name = g_strdup_printf (_("Screenshot-%d.png"), job->iteration);
+        }
+    }
+
+  retval = g_build_filename (job->base_uris[job->type], file_name, NULL);
+  g_free (file_name);
+
+  return retval;
+}
+
+static gboolean
+try_check_file (GIOSchedulerJob *io_job,
+                GCancellable *cancellable,
+                gpointer data)
+{
+  AsyncExistenceJob *job = data;
+  GFile *file;
+  GFileInfo *info;
+  GError *error;
+  char *uri;
+
+retry:
+  error = NULL;
+  uri = build_uri (job);
+  file = g_file_new_for_uri (uri);
+
+  info = g_file_query_info (file, G_FILE_ATTRIBUTE_STANDARD_TYPE,
+			    G_FILE_QUERY_INFO_NONE, cancellable, &error);
+  if (info != NULL)
+    {
+      /* file already exists, iterate again */
+      g_object_unref (info);
+      g_object_unref (file);
+      g_free (uri);
+
+      (job->iteration)++;
+
+      goto retry;
+    }
+  else
+    {
+      /* see the error to check whether the location is not accessible
+       * or the file does not exist.
+       */
+      if (error->code == G_IO_ERROR_NOT_FOUND)
+        {
+          GFile *parent;
+
+          /* if the parent directory doesn't exist as well, forget the saved
+           * directory and treat this as a generic error.
+           */
+
+          parent = g_file_get_parent (file);
+
+          if (!g_file_query_exists (parent, NULL))
+            {
+              (job->type)++;
+              job->iteration = 0;
+
+              g_object_unref (file);
+              g_object_unref (parent);
+              goto retry;
+            }
+          else
+            {
+              job->retval = uri;
+
+              g_object_unref (parent);
+              goto out;
+            }
+        }
+      else
+        {
+          /* another kind of error, assume this location is not
+           * accessible.
+           */
+          g_free (uri);
+          if (job->type == TEST_TMP)
+            {
+              job->retval = NULL;
+              goto out;
+            }
+          else
+            {
+              (job->type)++;
+              job->iteration = 0;
+
+              g_error_free (error);
+              g_object_unref (file);
+              goto retry;
+            }
+        }
+    }
+
+out:
+  g_error_free (error);
+  g_object_unref (file);
+
+  g_io_scheduler_job_send_to_mainloop_async (io_job,
+                                             check_file_done,
+                                             job,
+                                             NULL);
+  return FALSE;
+}
+
+static GdkWindow *
+find_current_window (char **window_title)
+{
+  GdkWindow *window;
+
+  if (!screenshot_grab_lock ())
+    exit (0);
+
+  if (take_window_shot)
+    {
+      window = screenshot_find_current_window ();
+      if (!window)
+	{
+	  take_window_shot = FALSE;
+	  window = gdk_get_default_root_window ();
+	}
+      else
+	{
+	  gchar *tmp, *sanitized;
+
+	  tmp = screenshot_get_window_title (window);
+	  sanitized = screenshot_sanitize_filename (tmp);
+	  g_free (tmp);
+	  *window_title = sanitized;
+	}
+    }
+  else
+    {
+      window = gdk_get_default_root_window ();
+    }
+
+  return window;
+}
+
+static void
+prepare_screenshot (void)
+{
+  AsyncExistenceJob *job;
+  
+  job = g_slice_new0 (AsyncExistenceJob);
+  job->base_uris[0] = last_save_dir;
+  /* we'll have to free these two */
+  job->base_uris[1] = get_desktop_dir ();
+  job->base_uris[2] = g_strconcat ("file://", g_get_tmp_dir (), NULL);
+  job->iteration = 0;
+  job->type = TEST_LAST_DIR;
+  job->window = find_current_window (&window_title);
+
+  g_io_scheduler_push_job (try_check_file,
+                           job,
+                           NULL,
+                           0, NULL);
+                           
 }
 
 static gboolean
 prepare_screenshot_timeout (gpointer data)
 {
-  gtk_main_quit ();
   prepare_screenshot ();
+  save_options ();
 
   return FALSE;
 }
@@ -769,25 +988,51 @@ get_desktop_dir (void)
                                                "/apps/nautilus/preferences/desktop_is_home_dir",
                                                NULL);
   if (desktop_is_home_dir)
-    desktop_dir = g_build_filename (g_get_home_dir (), NULL);
+    desktop_dir = g_strconcat ("file://", g_get_home_dir (), NULL);
   else
-    desktop_dir = g_build_filename (g_get_home_dir (), "Desktop", NULL);
+    desktop_dir = g_strconcat ("file://", g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP), NULL);
 
   g_object_unref (gconf_client);
 
   return desktop_dir;
 }
 
+/* Taken from gnome-vfs-utils.c */
+static char *
+expand_initial_tilde (const char *path)
+{
+  char *slash_after_user_name, *user_name;
+  struct passwd *passwd_file_entry;
+
+  if (path[1] == '/' || path[1] == '\0') {
+    return g_strconcat (g_get_home_dir (), &path[1], NULL);
+  }
+  
+  slash_after_user_name = strchr (&path[1], '/');
+  if (slash_after_user_name == NULL) {
+    user_name = g_strdup (&path[1]);
+  } else {
+    user_name = g_strndup (&path[1],
+                           slash_after_user_name - &path[1]);
+  }
+  passwd_file_entry = getpwnam (user_name);
+  g_free (user_name);
+  
+  if (passwd_file_entry == NULL || passwd_file_entry->pw_dir == NULL) {
+    return g_strdup (path);
+  }
+  
+  return g_strconcat (passwd_file_entry->pw_dir,
+                      slash_after_user_name,
+                      NULL);
+}
+
 /* Load options */
 static void
 load_options (void)
 {
-  GnomeClient *client;
   GConfClient *gconf_client;
 
-  client = gnome_master_client ();
-  gnome_client_set_restart_style (client, GNOME_RESTART_NEVER);
-
   gconf_client = gconf_client_get_default ();
 
   /* Find various dirs */
@@ -800,7 +1045,7 @@ load_options (void)
     }
   else if (last_save_dir[0] == '~')
     {
-      char *tmp = gnome_vfs_expand_initial_tilde (last_save_dir);
+      char *tmp = expand_initial_tilde (last_save_dir);
       g_free (last_save_dir);
       last_save_dir = tmp;
     }
@@ -809,20 +1054,50 @@ load_options (void)
                                           INCLUDE_BORDER_KEY,
                                           NULL);
 
+  include_pointer = gconf_client_get_bool (gconf_client,
+                                          INCLUDE_POINTER_KEY,
+                                          NULL);
+
   border_effect = gconf_client_get_string (gconf_client,
                                            BORDER_EFFECT_KEY,
                                            NULL);
   if (!border_effect)
     border_effect = g_strdup ("none");
 
+  delay = gconf_client_get_int (gconf_client, DELAY_KEY, NULL);
+
   g_object_unref (gconf_client);
 }
 
+static void
+save_options (void)
+{
+  GConfClient *gconf_client;
+
+  gconf_client = gconf_client_get_default ();
+
+  /* Error is NULL, as there's nothing we can do */
+
+  gconf_client_set_bool (gconf_client,
+                         INCLUDE_BORDER_KEY, include_border,
+                         NULL);
+  gconf_client_set_bool (gconf_client,
+                         INCLUDE_POINTER_KEY, include_pointer,
+                         NULL);
+  gconf_client_set_int (gconf_client, DELAY_KEY, delay, NULL);
+  gconf_client_set_string (gconf_client,
+                           BORDER_EFFECT_KEY, border_effect,
+                           NULL);
+ 
+  g_object_unref (gconf_client);
+}
+
+
 static void
 register_screenshooter_icon (GtkIconFactory * factory)
 {
-  GtkIconSource * source;
-  GtkIconSet * icon_set;
+  GtkIconSource *source;
+  GtkIconSet *icon_set;
 
   source = gtk_icon_source_new ();
   gtk_icon_source_set_icon_name (source, SCREENSHOOTER_ICON);
@@ -838,12 +1113,8 @@ register_screenshooter_icon (GtkIconFactory * factory)
 static void
 screenshooter_init_stock_icons (void)
 {
-  GtkIconFactory * factory;
-  GtkIconSize screenshooter_icon_size;
+  GtkIconFactory *factory;
 
-  screenshooter_icon_size = gtk_icon_size_register ("panel-menu", 
-	                                            GTK_ICON_SIZE_DIALOG,
-	                                            GTK_ICON_SIZE_DIALOG);
   factory = gtk_icon_factory_new ();
   gtk_icon_factory_add_default (factory);
 
@@ -855,18 +1126,20 @@ screenshooter_init_stock_icons (void)
 int
 main (int argc, char *argv[])
 {
-  GnomeProgram *program;
   GOptionContext *context;
   GOptionGroup *group;
   gboolean window_arg = FALSE;
   gboolean include_border_arg = FALSE;
+  gboolean disable_border_arg = FALSE;
   gboolean interactive_arg = FALSE;
   gchar *border_effect_arg = NULL;
   guint delay_arg = 0;
+  GError *error = NULL;
 
   const GOptionEntry entries[] = {
     { "window", 'w', 0, G_OPTION_ARG_NONE, &window_arg, N_("Grab a window instead of the entire screen"), NULL },
     { "include-border", 'b', 0, G_OPTION_ARG_NONE, &include_border_arg, N_("Include the window border with the screenshot"), NULL },
+    { "remove-border", 'B', 0, G_OPTION_ARG_NONE, &disable_border_arg, N_("Remove the window border from the screenshot"), NULL },
     { "delay", 'd', 0, G_OPTION_ARG_INT, &delay_arg, N_("Take screenshot after specified delay [in seconds]"), N_("seconds") },
     { "border-effect", 'e', 0, G_OPTION_ARG_STRING, &border_effect_arg, N_("Effect to add to the border (shadow, border or none)"), N_("effect") },
     { "interactive", 'i', 0, G_OPTION_ARG_NONE, &interactive_arg, N_("Interactively set options"), NULL },
@@ -878,26 +1151,23 @@ main (int argc, char *argv[])
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 
-  group = g_option_group_new ("gnome-screenshot",
-		  	      _("Options for Screenshot"),
-			      _("Show Screenshot options"),
-			      NULL, NULL);
-  g_option_group_add_entries (group, entries);
-
   context = g_option_context_new (_("Take a picture of the screen"));
   g_option_context_set_ignore_unknown_options (context, FALSE);
   g_option_context_set_help_enabled (context, TRUE);
-  g_option_context_set_main_group (context, group);
+  g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
+  g_option_context_add_group (context, gtk_get_option_group (TRUE));
 
-  gnome_authentication_manager_init ();
+  g_option_context_parse (context, &argc, &argv, &error);
+
+  if (error) {
+    g_critical ("Unable to parse arguments: %s", error->message);
+    g_error_free (error);
+    g_option_context_free (context);
+    exit (1);
+  }
+
+  g_option_context_free (context);
 
-  program = gnome_program_init ("gnome-screenshot", VERSION,
-				LIBGNOMEUI_MODULE,
-				argc, argv,
-				GNOME_PARAM_GOPTION_CONTEXT, context,
-				GNOME_PROGRAM_STANDARD_PROPERTIES,
-				NULL);
-  glade_gnome_init();
   gtk_window_set_default_icon_name (SCREENSHOOTER_ICON);
   screenshooter_init_stock_icons ();
 
@@ -909,6 +1179,9 @@ main (int argc, char *argv[])
   if (include_border_arg)
     include_border = TRUE;
 
+  if (disable_border_arg)
+    include_border = FALSE;
+
   if (border_effect_arg)
     {
       g_free (border_effect);
@@ -932,7 +1205,6 @@ main (int argc, char *argv[])
         {
         case GTK_RESPONSE_DELETE_EVENT:
         case GTK_RESPONSE_CANCEL:
-          g_object_unref (program);
           return EXIT_SUCCESS;
         case GTK_RESPONSE_OK:
           break;
@@ -942,19 +1214,19 @@ main (int argc, char *argv[])
         }
     }
 
-  if (delay > 0)
+  if ((delay > 0 && interactive_arg) || delay_arg > 0)
     {      
       g_timeout_add (delay * 1000,
 		     prepare_screenshot_timeout,
 		     NULL);
-      gtk_main ();
     }
   else
     {
-      prepare_screenshot ();
+      /* start this in an idle anyway and fire up the mainloop */
+      g_idle_add (prepare_screenshot_timeout, NULL);
     }
 
-  g_object_unref (program);
+  gtk_main ();
 
   return EXIT_SUCCESS;
 }
